
from django.urls import reverse
from rest_framework import status

from arkindex.process.models import FeatureUsage, Worker
from arkindex.project.tests import FixtureAPITestCase


class TestWorkerVersionRetrieve(FixtureAPITestCase):

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.worker_reco = Worker.objects.get(slug="reco")
        cls.version_1 = cls.worker_reco.versions.get()

    def test_retrieve_invalid_id(self):
        self.client.force_login(self.user)
        response = self.client.get(
            reverse("api:version-retrieve", kwargs={"pk": "12341234-1234-1234-1234-123412341234"})
        )
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_retrieve_no_login(self):
        """
        Worker versions may be retrieved with no authentication in order
        to see the version on public resources
        """
        with self.assertNumQueries(2):
            response = self.client.get(reverse("api:version-retrieve", kwargs={"pk": str(self.version_1.id)}))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        data = response.json()
        self.assertDictEqual(data, {
            "id": str(self.version_1.id),
            "configuration": {"test": 42},
            "docker_image_iid": self.version_1.docker_image_iid,
            "state": "available",
            "gpu_usage": "disabled",
            "model_usage": FeatureUsage.Disabled.value,
            "revision_url": None,
            "version": 1,
            "tag": None,
            "branch": None,
            "worker": {
                "id": str(self.worker_reco.id),
                "name": "Recognizer",
                "type": "recognizer",
                "slug": "reco",
                "description": "",
                "archived": False,
                "repository_url": None,
            },
            "created": "2020-02-02T01:23:45.678000Z",
        })

    def test_retrieve(self):
        self.client.force_login(self.user)
        response = self.client.get(reverse("api:version-retrieve", kwargs={"pk": str(self.version_1.id)}))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        data = response.json()
        self.assertDictEqual(data, {
            "id": str(self.version_1.id),
            "configuration": {"test": 42},
            "docker_image_iid": self.version_1.docker_image_iid,
            "state": "available",
            "gpu_usage": "disabled",
            "model_usage": FeatureUsage.Disabled.value,
            "revision_url": None,
            "version": 1,
            "tag": None,
            "branch": None,
            "worker": {
                "id": str(self.worker_reco.id),
                "name": "Recognizer",
                "type": "recognizer",
                "slug": "reco",
                "description": "",
                "archived": False,
                "repository_url": None,
            },
            "created": "2020-02-02T01:23:45.678000Z",
        })
